# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from pygfe.models.base_model_ import Model
from seqann import util


class Error(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, message: str=None, pygfe_version: str=None, gfedb_version: str=None, imgtdb_version: str=None, log: List[str]=None, type: str=None):  # noqa: E501
        """Error - a model defined in Swagger

        :param message: The message of this Error.  # noqa: E501
        :type message: str
        :param pygfe_version: The pygfe_version of this Error.  # noqa: E501
        :type pygfe_version: str
        :param gfedb_version: The gfedb_version of this Error.  # noqa: E501
        :type gfedb_version: str
        :param imgtdb_version: The imgtdb_version of this Error.  # noqa: E501
        :type imgtdb_version: str
        :param log: The log of this Error.  # noqa: E501
        :type log: List[str]
        :param type: The type of this Error.  # noqa: E501
        :type type: str
        """
        self.swagger_types = {
            'message': str,
            'pygfe_version': str,
            'gfedb_version': str,
            'imgtdb_version': str,
            'log': List[str],
            'type': str
        }

        self.attribute_map = {
            'message': 'Message',
            'pygfe_version': 'pygfe_version',
            'gfedb_version': 'gfedb_version',
            'imgtdb_version': 'imgtdb_version',
            'log': 'log',
            'type': 'type'
        }

        self._message = message
        self._pygfe_version = pygfe_version
        self._gfedb_version = gfedb_version
        self._imgtdb_version = imgtdb_version
        self._log = log
        self._type = type

    @classmethod
    def from_dict(cls, dikt) -> 'Error':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Error of this Error.  # noqa: E501
        :rtype: Error
        """
        return util.deserialize_model(dikt, cls)

    @property
    def message(self) -> str:
        """Gets the message of this Error.


        :return: The message of this Error.
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message: str):
        """Sets the message of this Error.


        :param message: The message of this Error.
        :type message: str
        """
        if message is None:
            raise ValueError("Invalid value for `message`, must not be `None`")  # noqa: E501

        self._message = message

    @property
    def pygfe_version(self) -> str:
        """Gets the pygfe_version of this Error.


        :return: The pygfe_version of this Error.
        :rtype: str
        """
        return self._pygfe_version

    @pygfe_version.setter
    def pygfe_version(self, pygfe_version: str):
        """Sets the pygfe_version of this Error.


        :param pygfe_version: The pygfe_version of this Error.
        :type pygfe_version: str
        """
        if pygfe_version is None:
            raise ValueError("Invalid value for `pygfe_version`, must not be `None`")  # noqa: E501

        self._pygfe_version = pygfe_version

    @property
    def gfedb_version(self) -> str:
        """Gets the gfedb_version of this Error.


        :return: The gfedb_version of this Error.
        :rtype: str
        """
        return self._gfedb_version

    @gfedb_version.setter
    def gfedb_version(self, gfedb_version: str):
        """Sets the gfedb_version of this Error.


        :param gfedb_version: The gfedb_version of this Error.
        :type gfedb_version: str
        """
        if gfedb_version is None:
            raise ValueError("Invalid value for `gfedb_version`, must not be `None`")  # noqa: E501

        self._gfedb_version = gfedb_version

    @property
    def imgtdb_version(self) -> str:
        """Gets the imgtdb_version of this Error.


        :return: The imgtdb_version of this Error.
        :rtype: str
        """
        return self._imgtdb_version

    @imgtdb_version.setter
    def imgtdb_version(self, imgtdb_version: str):
        """Sets the imgtdb_version of this Error.


        :param imgtdb_version: The imgtdb_version of this Error.
        :type imgtdb_version: str
        """
        if imgtdb_version is None:
            raise ValueError("Invalid value for `imgtdb_version`, must not be `None`")  # noqa: E501

        self._imgtdb_version = imgtdb_version

    @property
    def log(self) -> List[str]:
        """Gets the log of this Error.


        :return: The log of this Error.
        :rtype: List[str]
        """
        return self._log

    @log.setter
    def log(self, log: List[str]):
        """Sets the log of this Error.


        :param log: The log of this Error.
        :type log: List[str]
        """

        self._log = log

    @property
    def type(self) -> str:
        """Gets the type of this Error.


        :return: The type of this Error.
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type: str):
        """Sets the type of this Error.


        :param type: The type of this Error.
        :type type: str
        """

        self._type = type



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from pygfe.models.base_model_ import Model
from seqann import util


class Seqdiff(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, term: str=None, rank: str=None, location: str=None, ref: str=None, inseq: str=None):  # noqa: E501
        """Seqdiff - a model defined in Swagger

        :param term: The term of this Seqdiff.  # noqa: E501
        :type term: str
        :param rank: The rank of this Seqdiff.  # noqa: E501
        :type rank: str
        :param location: The location of this Seqdiff.  # noqa: E501
        :type location: str
        :param ref: The ref of this Seqdiff.  # noqa: E501
        :type ref: str
        :param inseq: The inseq of this Seqdiff.  # noqa: E501
        :type inseq: str
        """
        self.swagger_types = {
            'term': str,
            'rank': str,
            'location': str,
            'ref': str,
            'inseq': str
        }

        self.attribute_map = {
            'term': 'term',
            'rank': 'rank',
            'location': 'location',
            'ref': 'ref',
            'inseq': 'inseq'
        }

        self._term = term
        self._rank = rank
        self._location = location
        self._ref = ref
        self._inseq = inseq

    @classmethod
    def from_dict(cls, dikt) -> 'Seqdiff':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Seqdiff of this Seqdiff.  # noqa: E501
        :rtype: Seqdiff
        """
        return util.deserialize_model(dikt, cls)

    @property
    def term(self) -> str:
        """Gets the term of this Seqdiff.


        :return: The term of this Seqdiff.
        :rtype: str
        """
        return self._term

    @term.setter
    def term(self, term: str):
        """Sets the term of this Seqdiff.


        :param term: The term of this Seqdiff.
        :type term: str
        """

        self._term = term

    @property
    def rank(self) -> str:
        """Gets the rank of this Seqdiff.


        :return: The rank of this Seqdiff.
        :rtype: str
        """
        return self._rank

    @rank.setter
    def rank(self, rank: str):
        """Sets the rank of this Seqdiff.


        :param rank: The rank of this Seqdiff.
        :type rank: str
        """

        self._rank = rank

    @property
    def location(self) -> str:
        """Gets the location of this Seqdiff.


        :return: The location of this Seqdiff.
        :rtype: str
        """
        return self._location

    @location.setter
    def location(self, location: str):
        """Sets the location of this Seqdiff.


        :param location: The location of this Seqdiff.
        :type location: str
        """

        self._location = location

    @property
    def ref(self) -> str:
        """Gets the ref of this Seqdiff.


        :return: The ref of this Seqdiff.
        :rtype: str
        """
        return self._ref

    @ref.setter
    def ref(self, ref: str):
        """Sets the ref of this Seqdiff.


        :param ref: The ref of this Seqdiff.
        :type ref: str
        """

        self._ref = ref

    @property
    def inseq(self) -> str:
        """Gets the inseq of this Seqdiff.


        :return: The inseq of this Seqdiff.
        :rtype: str
        """
        return self._inseq

    @inseq.setter
    def inseq(self, inseq: str):
        """Sets the inseq of this Seqdiff.


        :param inseq: The inseq of this Seqdiff.
        :type inseq: str
        """

        self._inseq = inseq



# coding: utf-8

"""

    No descripton provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
import re


class FeatureRequest(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, locus=None, term=None, rank=None, sequence=None):
        """
        FeatureRequest - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'locus': 'str',
            'term': 'str',
            'rank': 'int',
            'sequence': 'str'
        }

        self.attribute_map = {
            'locus': 'locus',
            'term': 'term',
            'rank': 'rank',
            'sequence': 'sequence'
        }

        self._locus = locus
        self._term = term
        self._rank = rank
        self._sequence = sequence

    @property
    def locus(self):
        """
        Gets the locus of this FeatureRequest.
        locus name or URI

        :return: The locus of this FeatureRequest.
        :rtype: str
        """
        return self._locus

    @locus.setter
    def locus(self, locus):
        """
        Sets the locus of this FeatureRequest.
        locus name or URI

        :param locus: The locus of this FeatureRequest.
        :type: str
        """

        self._locus = locus

    @property
    def term(self):
        """
        Gets the term of this FeatureRequest.
        Sequence Ontology (SO) term name, accession, or URI

        :return: The term of this FeatureRequest.
        :rtype: str
        """
        return self._term

    @term.setter
    def term(self, term):
        """
        Sets the term of this FeatureRequest.
        Sequence Ontology (SO) term name, accession, or URI

        :param term: The term of this FeatureRequest.
        :type: str
        """

        self._term = term

    @property
    def rank(self):
        """
        Gets the rank of this FeatureRequest.
        feature rank, must be at least 1

        :return: The rank of this FeatureRequest.
        :rtype: int
        """
        return self._rank

    @rank.setter
    def rank(self, rank):
        """
        Sets the rank of this FeatureRequest.
        feature rank, must be at least 1

        :param rank: The rank of this FeatureRequest.
        :type: int
        """

        self._rank = rank

    @property
    def sequence(self):
        """
        Gets the sequence of this FeatureRequest.
        feature sequence, in DNA alphabet

        :return: The sequence of this FeatureRequest.
        :rtype: str
        """
        return self._sequence

    @sequence.setter
    def sequence(self, sequence):
        """
        Sets the sequence of this FeatureRequest.
        feature sequence, in DNA alphabet

        :param sequence: The sequence of this FeatureRequest.
        :type: str
        """

        self._sequence = sequence

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

"""

    No descripton provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from __future__ import absolute_import

import sys
import os
import re

# python 2 and python 3 compatibility library
from six import iteritems

from ..configuration import Configuration
from ..api_client import ApiClient


class FeaturesApi(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        config = Configuration()
        if api_client:
            self.api_client = api_client
        else:
            if not config.api_client:
                config.api_client = ApiClient()
            self.api_client = config.api_client

    def create_feature(self, **kwargs):
        """
        Create an enumerated sequence feature
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.create_feature(callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param FeatureRequest body: 
        :return: Feature
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.create_feature_with_http_info(**kwargs)
        else:
            (data) = self.create_feature_with_http_info(**kwargs)
            return data

    def create_feature_with_http_info(self, **kwargs):
        """
        Create an enumerated sequence feature
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.create_feature_with_http_info(callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param FeatureRequest body: 
        :return: Feature
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']
        all_params.append('callback')
        all_params.append('_return_http_data_only')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method create_feature" % key
                )
            params[key] = val
        del params['kwargs']

        resource_path = '/features'.replace('{format}', 'json')
        path_params = {}

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'body' in params:
            body_params = params['body']

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type(['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'POST',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='Feature',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'))

    def get_feature_by_path(self, locus, term, rank, accession, **kwargs):
        """
        Retrieve an enumerated sequence feature
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.get_feature_by_path(locus, term, rank, accession, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI (required)
        :param str term: Sequence Ontology (SO) term name, accession, or URI (required)
        :param int rank: feature rank, must be at least 1 (required)
        :param int accession: accession, must be at least 1 (required)
        :return: Feature
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.get_feature_by_path_with_http_info(locus, term, rank, accession, **kwargs)
        else:
            (data) = self.get_feature_by_path_with_http_info(locus, term, rank, accession, **kwargs)
            return data

    def get_feature_by_path_with_http_info(self, locus, term, rank, accession, **kwargs):
        """
        Retrieve an enumerated sequence feature
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.get_feature_by_path_with_http_info(locus, term, rank, accession, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI (required)
        :param str term: Sequence Ontology (SO) term name, accession, or URI (required)
        :param int rank: feature rank, must be at least 1 (required)
        :param int accession: accession, must be at least 1 (required)
        :return: Feature
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['locus', 'term', 'rank', 'accession']
        all_params.append('callback')
        all_params.append('_return_http_data_only')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_feature_by_path" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'locus' is set
        if ('locus' not in params) or (params['locus'] is None):
            raise ValueError("Missing the required parameter `locus` when calling `get_feature_by_path`")
        # verify the required parameter 'term' is set
        if ('term' not in params) or (params['term'] is None):
            raise ValueError("Missing the required parameter `term` when calling `get_feature_by_path`")
        # verify the required parameter 'rank' is set
        if ('rank' not in params) or (params['rank'] is None):
            raise ValueError("Missing the required parameter `rank` when calling `get_feature_by_path`")
        # verify the required parameter 'accession' is set
        if ('accession' not in params) or (params['accession'] is None):
            raise ValueError("Missing the required parameter `accession` when calling `get_feature_by_path`")

        resource_path = '/features/{locus}/{term}/{rank}/{accession}'.replace('{format}', 'json')
        path_params = {}
        if 'locus' in params:
            path_params['locus'] = params['locus']
        if 'term' in params:
            path_params['term'] = params['term']
        if 'rank' in params:
            path_params['rank'] = params['rank']
        if 'accession' in params:
            path_params['accession'] = params['accession']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type(['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='Feature',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'))

    def get_feature_by_query(self, **kwargs):
        """
        Retrieve an enumerated sequence feature
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.get_feature_by_query(callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI
        :param str term: Sequence Ontology (SO) term name, accession, or URI
        :param int rank: feature rank, must be at least 1
        :param int accession: accession, must be at least 1
        :return: Feature
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.get_feature_by_query_with_http_info(**kwargs)
        else:
            (data) = self.get_feature_by_query_with_http_info(**kwargs)
            return data

    def get_feature_by_query_with_http_info(self, **kwargs):
        """
        Retrieve an enumerated sequence feature
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.get_feature_by_query_with_http_info(callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI
        :param str term: Sequence Ontology (SO) term name, accession, or URI
        :param int rank: feature rank, must be at least 1
        :param int accession: accession, must be at least 1
        :return: Feature
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['locus', 'term', 'rank', 'accession']
        all_params.append('callback')
        all_params.append('_return_http_data_only')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_feature_by_query" % key
                )
            params[key] = val
        del params['kwargs']

        resource_path = '/features'.replace('{format}', 'json')
        path_params = {}

        query_params = {}
        if 'locus' in params:
            query_params['locus'] = params['locus']
        if 'term' in params:
            query_params['term'] = params['term']
        if 'rank' in params:
            query_params['rank'] = params['rank']
        if 'accession' in params:
            query_params['accession'] = params['accession']

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type(['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='Feature',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'))

    def list_features(self, locus, **kwargs):
        """
        List the enumerated sequence features at a locus
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.list_features(locus, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI (required)
        :return: list[Feature]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.list_features_with_http_info(locus, **kwargs)
        else:
            (data) = self.list_features_with_http_info(locus, **kwargs)
            return data

    def list_features_with_http_info(self, locus, **kwargs):
        """
        List the enumerated sequence features at a locus
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.list_features_with_http_info(locus, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI (required)
        :return: list[Feature]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['locus']
        all_params.append('callback')
        all_params.append('_return_http_data_only')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_features" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'locus' is set
        if ('locus' not in params) or (params['locus'] is None):
            raise ValueError("Missing the required parameter `locus` when calling `list_features`")

        resource_path = '/features/{locus}'.replace('{format}', 'json')
        path_params = {}
        if 'locus' in params:
            path_params['locus'] = params['locus']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type(['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='list[Feature]',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'))

    def list_features_0(self, locus, term, **kwargs):
        """
        List the enumerated sequence features matching a term at a locus
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.list_features_0(locus, term, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI (required)
        :param str term: Sequence Ontology (SO) term name, accession, or URI (required)
        :return: list[Feature]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.list_features_0_with_http_info(locus, term, **kwargs)
        else:
            (data) = self.list_features_0_with_http_info(locus, term, **kwargs)
            return data

    def list_features_0_with_http_info(self, locus, term, **kwargs):
        """
        List the enumerated sequence features matching a term at a locus
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.list_features_0_with_http_info(locus, term, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI (required)
        :param str term: Sequence Ontology (SO) term name, accession, or URI (required)
        :return: list[Feature]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['locus', 'term']
        all_params.append('callback')
        all_params.append('_return_http_data_only')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_features_0" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'locus' is set
        if ('locus' not in params) or (params['locus'] is None):
            raise ValueError("Missing the required parameter `locus` when calling `list_features_0`")
        # verify the required parameter 'term' is set
        if ('term' not in params) or (params['term'] is None):
            raise ValueError("Missing the required parameter `term` when calling `list_features_0`")

        resource_path = '/features/{locus}/{term}'.replace('{format}', 'json')
        path_params = {}
        if 'locus' in params:
            path_params['locus'] = params['locus']
        if 'term' in params:
            path_params['term'] = params['term']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type(['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='list[Feature]',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'))

    def list_features_1(self, locus, term, rank, **kwargs):
        """
        List the enumerated sequence features matching a term and rank at a locus
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.list_features_1(locus, term, rank, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI (required)
        :param str term: Sequence Ontology (SO) term name, accession, or URI (required)
        :param int rank: feature rank, must be at least 1 (required)
        :return: list[Feature]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('callback'):
            return self.list_features_1_with_http_info(locus, term, rank, **kwargs)
        else:
            (data) = self.list_features_1_with_http_info(locus, term, rank, **kwargs)
            return data

    def list_features_1_with_http_info(self, locus, term, rank, **kwargs):
        """
        List the enumerated sequence features matching a term and rank at a locus
        

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please define a `callback` function
        to be invoked when receiving the response.
        >>> def callback_function(response):
        >>>     pprint(response)
        >>>
        >>> thread = api.list_features_1_with_http_info(locus, term, rank, callback=callback_function)

        :param callback function: The callback function
            for asynchronous request. (optional)
        :param str locus: locus name or URI (required)
        :param str term: Sequence Ontology (SO) term name, accession, or URI (required)
        :param int rank: feature rank, must be at least 1 (required)
        :return: list[Feature]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['locus', 'term', 'rank']
        all_params.append('callback')
        all_params.append('_return_http_data_only')

        params = locals()
        for key, val in iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method list_features_1" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'locus' is set
        if ('locus' not in params) or (params['locus'] is None):
            raise ValueError("Missing the required parameter `locus` when calling `list_features_1`")
        # verify the required parameter 'term' is set
        if ('term' not in params) or (params['term'] is None):
            raise ValueError("Missing the required parameter `term` when calling `list_features_1`")
        # verify the required parameter 'rank' is set
        if ('rank' not in params) or (params['rank'] is None):
            raise ValueError("Missing the required parameter `rank` when calling `list_features_1`")

        resource_path = '/features/{locus}/{term}/{rank}'.replace('{format}', 'json')
        path_params = {}
        if 'locus' in params:
            path_params['locus'] = params['locus']
        if 'term' in params:
            path_params['term'] = params['term']
        if 'rank' in params:
            path_params['rank'] = params['rank']

        query_params = {}

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None

        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.\
            select_header_accept(['application/json'])
        if not header_params['Accept']:
            del header_params['Accept']

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.\
            select_header_content_type(['application/json'])

        # Authentication setting
        auth_settings = []

        return self.api_client.call_api(resource_path, 'GET',
                                            path_params,
                                            query_params,
                                            header_params,
                                            body=body_params,
                                            post_params=form_params,
                                            files=local_var_files,
                                            response_type='list[Feature]',
                                            auth_settings=auth_settings,
                                            callback=params.get('callback'),
                                            _return_http_data_only=params.get('_return_http_data_only'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#    pygfe pyGFE.
#    Copyright (c) 2017 Be The Match operated by National Marrow Donor Program. All Rights Reserved.
#
#    This library is free software; you can redistribute it and/or modify it
#    under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation; either version 3 of the License, or (at
#    your option) any later version.
#
#    This library is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; with out even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
#    License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this library;  if not, write to the Free Software Foundation,
#    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
#
#    > http://www.fsf.org/licensing/licenses/lgpl.html
#    > http://www.opensource.org/licenses/lgpl-license.php
#



"""
test_pygfe
----------------------------------

Tests for `pygfe` module.
"""


import sys
import unittest
#from pygfe.models.allele_call import AlleleCall
from pygfe.pygfe import pyGFE
import os

from Bio import SeqIO
#from pygfe.act import ACT
from py2neo import Graph
#from pygfe.gfedb import GfeDB
#from pygfe.gfe import GFE
from pygfe.models.error import Error
#from pygfe.models.feature import Feature
from pygfe.models.typing import Typing
from pygfe.models.feature import Feature
from pygfe.models.seqdiff import Seqdiff

from seqann.sequence_annotation import BioSeqAnn
from BioSQL import BioSeqDatabase

import pymysql
import pandas as pd
from pygfe.cypher import all_feats
from pygfe.cypher import all_gfe2hla
import pickle
import time

neo4jpass = 'gfedb'
if os.getenv("NEO4JPASS"):
    neo4jpass = os.getenv("NEO4JPASS")

neo4juser = 'neo4j'
if os.getenv("NEO4JUSER"):
    neo4juser = os.getenv("NEO4JUSER")

neo4jurl = "http://neo4j.b12x.org:80"
if os.getenv("NEO4JURL"):
    neo4jurl = os.getenv("NEO4JURL")

biosqlpass = "my-secret-pw"
if os.getenv("BIOSQLPASS"):
    biosqlpass = os.getenv("BIOSQLPASS")

biosqluser = 'root'
if os.getenv("BIOSQLUSER"):
    biosqluser = os.getenv("BIOSQLUSER")

biosqlhost = "localhost"
if os.getenv("BIOSQLHOST"):
    biosqlhost = os.getenv("BIOSQLHOST")

biosqldb = "bioseqdb"
if os.getenv("BIOSQLDB"):
    biosqldb = os.getenv("BIOSQLDB")

biosqlport = 3306
if os.getenv("BIOSQLPORT"):
    biosqlport = os.getenv("BIOSQLPORT")

import logging
logging.basicConfig(format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                    datefmt='%m/%d/%Y %I:%M:%S %p',
                    level=logging.INFO)


def conn():
    try:
        # print(biosqlpass, biosqluser, biosqlhost, biosqldb, biosqlport, sep="\t")
        conn = pymysql.connect(host=biosqlhost,
                               port=biosqlport, user=biosqluser,
                               passwd=biosqlpass, db=biosqldb)
        conn.close()
        return True
    except Exception as e:
        print("Exception while checking MYSQL Connection:" + str(e))
        return False


class TestPygfe(unittest.TestCase):

    def setUp(self):
        self.data_dir = os.path.dirname(__file__) + "/resources"
        pass

    def tearDown(self):
        pass

    def test_000_pygfe(self):
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb)
        seqann = BioSeqAnn(server=server, verbose=False)
        seqann = "X"
        #else:
        #    print
        #    seqann = BioSeqAnn()
        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      load_features=False,
                      verbose=False,
                      load_all=True,
                      loci=["HLA-A"])
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/unknown_A.fasta", "fasta"))
        typing = pygfe.type_from_seq("HLA-A", str(seqs[1].seq))
        #self.assertEqual(typing.gfe, 'HLA-Aw770-1-1-1-1-1-1-1-1-1-1-1-1-1-1-1-4')
        self.assertEqual(typing.hla, 'HLA-A*01:01:01:01')
        self.assertEqual(typing.status, "novel")
        self.assertIsInstance(typing, Typing)
        pass

    def test_001_pygfe(self):
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():

        pickle_file1 = "unique_db-feats.pickle"
        pickle_file2 = "feature-service.pickle"
        pickle_gfe2feat = "gfe2feat.pickle"
        pickle_file3 = "gfe2hla.pickle"
        pickle_file4 = "seq2hla.pickle"

        with open(pickle_gfe2feat, 'rb') as handle1:
            gfe_feats = pickle.load(handle1)

        with open(pickle_file1, 'rb') as handle1:
            feats = pickle.load(handle1)

        with open(pickle_file2, 'rb') as handle2:
            cached_feats = pickle.load(handle2)

        with open(pickle_file3, 'rb') as handle3:
            gfe2hla = pickle.load(handle3)

        with open(pickle_file4, 'rb') as handle:
            seq2hla = pickle.load(handle)

        seqann = BioSeqAnn(verbose=False, cached_features=cached_feats, align=True)

        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      gfe_feats=gfe_feats,
                      gfe2hla=gfe2hla,
                      seq2hla=seq2hla,
                      features=feats,
                      verbose=False)
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/unknown_A.fasta", "fasta"))
        typing = pygfe.type_from_seq("HLA-A", str(seqs[1].seq))
        print(typing)
        #self.assertEqual(typing.gfe, 'HLA-Aw770-1-1-1-1-1-1-1-1-1-1-1-1-1-1-1-4')
        self.assertEqual(typing.hla, 'HLA-A*01:01:01:01')
        self.assertEqual(typing.status, "novel")
        self.assertIsInstance(typing, Typing)
        pass

    def test_002_pygfe(self):
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():

        pickle_file1 = "unique_db-feats.pickle"
        pickle_file2 = "feature-service.pickle"
        pickle_gfe2feat = "gfe2feat.pickle"
        #pickle_file3 = "gfe2hla.pickle"
        pickle_file4 = "seq2hla.pickle"

        with open(pickle_gfe2feat, 'rb') as handle1:
            gfe_feats = pickle.load(handle1)

        with open(pickle_file1, 'rb') as handle1:
            feats = pickle.load(handle1)

        with open(pickle_file2, 'rb') as handle2:
            cached_feats = pickle.load(handle2)

        # with open(pickle_file3, 'rb') as handle3:
        #     gfe2hla = pickle.load(handle3)

        with open(pickle_file4, 'rb') as handle:
            seq2hla = pickle.load(handle)

        seqann = BioSeqAnn(verbose=False, cached_features=cached_feats, align=True)

        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      gfe_feats=gfe_feats,
                      seq2hla=seq2hla,
                      features=feats,
                      verbose=False)
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/unknown_A.fasta", "fasta"))
        typing = pygfe.type_from_seq("HLA-A", str(seqs[1].seq))
        print(typing)
        #self.assertEqual(typing.gfe, 'HLA-Aw770-1-1-1-1-1-1-1-1-1-1-1-1-1-1-1-4')
        self.assertEqual(typing.hla, 'HLA-A*01:01:01:01')
        self.assertEqual(typing.status, "novel")
        self.assertIsInstance(typing, Typing)
        pass


    def test_005_picklefiles(self):
        graph = Graph("http://ec2-34-207-175-160.compute-1.amazonaws.com:80", user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb,
                                              port=biosqlport)
        seqann = BioSeqAnn(server=server, verbose=False)

        gfe = GFE()
        #cached_feats = gfe.all_feats
        # print("Finished loading cached_feats")
        # pickle_service = "feature-service.pickle"
        # with open(pickle_service, 'wb') as handle2:
        #     pickle.dump(cached_feats, handle2, protocol=pickle.HIGHEST_PROTOCOL)

        feat_df = pd.DataFrame(graph.data(all_feats()))
        feat_df['ID'] = feat_df.apply(lambda row: ":".join([row['DB'],
                                                            row['LOC'],
                                                            str(row['RANK']),
                                                            row['TERM'],
                                                            row['SEQ']]),
                                      axis=1)
        feats = feat_df[['ID', 'ACCESSION']].set_index('ID').to_dict()['ACCESSION']

        print("Finished loading feats")
        pickle_feats = "unique_db-feats.pickle"
        with open(pickle_feats, 'wb') as handle1:
            pickle.dump(feats, handle1, protocol=pickle.HIGHEST_PROTOCOL)

        gfedb = GfeDB(graph=graph, persist=False, verbose=False)
        act = ACT(gfedb=gfedb, seqann=seqann, load_gfe2hla=True,
                  load_gfe2feat=True,
                  load_seq2hla=True, gfe=gfe)

        print("Finished loading all!!")

        gfe2hla = act.gfe2hla
        seq2hla = act.seq2hla
        gfe2feat = act.gfe_feats

        pickle_gfe2feat = "gfe2feat.pickle"
        with open(pickle_gfe2feat, 'wb') as handle5:
            pickle.dump(gfe2feat, handle5, protocol=pickle.HIGHEST_PROTOCOL)

        pickle_gfe2hla = "gfe2hla.pickle"
        with open(pickle_gfe2hla, 'wb') as handle3:
            pickle.dump(gfe2hla, handle3, protocol=pickle.HIGHEST_PROTOCOL)

        pickle_seq2hla = "seq2hla.pickle"
        with open(pickle_seq2hla, 'wb') as handle4:
            pickle.dump(seq2hla, handle4, protocol=pickle.HIGHEST_PROTOCOL)

        pass

    def test_003_loader1(self):
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb,
                                              port=3307)
        seqann = BioSeqAnn(server=server, verbose=True)

        pickle_file1 = "unique_db-feats.pickle"
        pickle_file2 = "feature-service.pickle"
        pickle_gfe2feat = "gfe2feat.pickle"
        pickle_file3 = "gfe2hla.pickle"
        pickle_file4 = "seq2hla.pickle"

        with open(pickle_gfe2feat, 'rb') as handle1:
            gfe_feats = pickle.load(handle1)

        with open(pickle_file1, 'rb') as handle1:
            feats = pickle.load(handle1)

        with open(pickle_file2, 'rb') as handle2:
            cached_feats = pickle.load(handle2)

        with open(pickle_file3, 'rb') as handle3:
            gfe2hla = pickle.load(handle3)

        with open(pickle_file4, 'rb') as handle:
            seq2hla = pickle.load(handle)

        start = time.time()
        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      load_features=False,
                      verbose=True,
                      features=feats,
                      seq2hla=seq2hla,
                      gfe2hla=gfe2hla,
                      gfe_feats=gfe_feats,
                      cached_features=cached_feats,
                      loci=["HLA-A"])
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/unknown_A.fasta", "fasta"))
        typing1 = pygfe.type_from_seq("HLA-A", str(seqs[1].seq), "3.31.0")
        typing2 = pygfe.type_from_seq("HLA-A", str(seqs[1].seq), "3.20.0")
        end = time.time()
        time_taken = end - start
        self.assertEqual(typing1.hla, 'HLA-A*01:01:01:01')
        self.assertEqual(typing1.status, "novel")
        self.assertIsInstance(typing1, Typing)
        self.assertEqual(typing2.hla, 'HLA-A*01:01:01:01')
        self.assertEqual(typing2.status, "novel")
        self.assertIsInstance(typing2, Typing)
        pass

    def test_003_loader2(self):
        start = time.time()
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb)
        seqann = BioSeqAnn(server=server, verbose=False)

        pickle_file1 = "unique_db-feats.pickle"
        pickle_file2 = "feature-service.pickle"
        pickle_gfe2feat = "gfe2feat.pickle"
        pickle_file3 = "gfe2hla.pickle"
        pickle_file4 = "seq2hla.pickle"
        with open(pickle_gfe2feat, 'rb') as handle1:
            gfe_feats = pickle.load(handle1)

        with open(pickle_file1, 'rb') as handle1:
            feats = pickle.load(handle1)

        with open(pickle_file2, 'rb') as handle2:
            cached_feats = pickle.load(handle2)

        with open(pickle_file3, 'rb') as handle3:
            gfe2hla = pickle.load(handle3)

        with open(pickle_file4, 'rb') as handle:
            seq2hla = pickle.load(handle)

        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      load_features=False,
                      verbose=False,
                      features=feats,
                      seq2hla=seq2hla,
                      gfe2hla=gfe2hla,
                      gfe_feats=gfe_feats,
                      cached_features=cached_feats,
                      loci=["HLA-A"])
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/known_A.fasta", "fasta"))
        typing1 = pygfe.type_from_seq("HLA-A", str(seqs[0].seq), "3.20.0")
        typing2 = pygfe.type_from_seq("HLA-A", str(seqs[0].seq), "3.31.0")
        end = time.time()
        time_taken = end - start
        print("TIME TAKEN: " + str(time_taken))
        self.assertEqual(typing2.hla, 'HLA-A*01:01:01:01')
        self.assertEqual(typing2.status, "documented")
        self.assertIsInstance(typing2, Typing)
        self.assertEqual(typing1.hla, 'HLA-A*01:01:01:01')
        self.assertEqual(typing1.status, "documented")
        self.assertIsInstance(typing1, Typing)
        pass

    def test_004_loader3(self):
        start = time.time()
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb, port=3307)
        seqann = BioSeqAnn(server=server, verbose=True)
        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      verbose=False,
                      load_features=False,
                      load_gfe2hla=True,
                      load_seq2hla=True,
                      load_gfe2feat=True,
                      loci=["HLA-A"])
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/known_A.fasta", "fasta"))
        #typing1 = pygfe.type_from_seq("HLA-A", str(seqs[0].seq), "3.20.0")
        typing2 = pygfe.type_from_seq("HLA-A", str(seqs[0].seq), "3.31.0")
        server.close()
        end = time.time()
        time_taken = end - start
        print("TIME TAKEN: " + str(time_taken))
        self.assertEqual(typing2.hla, 'HLA-A*01:01:01:01')
        self.assertEqual(typing2.status, "documented")
        self.assertIsInstance(typing2, Typing)
        # self.assertEqual(typing1.hla, 'HLA-A*01:01:01:01')
        # self.assertEqual(typing1.status, "documented")
        # self.assertIsInstance(typing1, Typing)
        pass

    def test_001_load_features(self):
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb,
                                              port=3307)
        seqann = BioSeqAnn(server=server)
        #else:
        #    seqann = BioSeqAnn()
        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      verbose=True,
                      load_features=True,
                      verbosity=2,
                      loci=["HLA-A"])
        self.assertIsInstance(pygfe, pyGFE)
        self.assertGreater(len(pygfe.gfe.structures), 1)
        self.assertGreater(len(pygfe.gfe.all_feats), 1)
        self.assertTrue('HLA-A' in pygfe.gfe.structures)
        self.assertFalse('HLA-Z' in pygfe.gfe.structures)
        pass

    def test_004_hml(self):
        #start = time.time()
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb,
                                              port=3307)
        seqann = BioSeqAnn(server=server, dbversion="3190", verbose=True)

        pickle_file1 = "unique_db-feats.pickle"
        pickle_file2 = "feature-service.pickle"
        pickle_gfe2feat = "gfe2feat.pickle"
        pickle_file3 = "gfe2hla.pickle"
        pickle_file4 = "seq2hla.pickle"
        with open(pickle_gfe2feat, 'rb') as handle1:
            gfe_feats = pickle.load(handle1)

        with open(pickle_file1, 'rb') as handle1:
            feats = pickle.load(handle1)

        with open(pickle_file2, 'rb') as handle2:
            cached_feats = pickle.load(handle2)

        with open(pickle_file3, 'rb') as handle3:
            gfe2hla = pickle.load(handle3)

        with open(pickle_file4, 'rb') as handle:
            seq2hla = pickle.load(handle)

        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      load_features=False,
                      verbose=True,
                      features=feats,
                      seq2hla=seq2hla,
                      gfe2hla=gfe2hla,
                      gfe_feats=gfe_feats,
                      cached_features=cached_feats,
                      loci=["HLA-C"])
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/hml_fail.fasta", "fasta"))
        typing1 = pygfe.type_from_seq("HLA-C", str(seqs[1].seq), "3.19.0")
        #typing2 = pygfe.type_from_seq("HLA-DRB1", str(seqs[0].seq), "3.31.0")
        #typing2 = pygfe.type_from_seq("HLA-DRB1", str(seqs[0].seq), "3.31.0")
        #end = time.time()
        #time_taken = end - start
        print(typing1)
        #print("=====")
        #print(typing2)
        # self.assertEqual(typing2.hla, 'HLA-A*01:01:01:01')
        # self.assertEqual(typing2.status, "documented")
        #self.assertIsInstance(typing2, Typing)
        # self.assertEqual(typing1.hla, 'HLA-A*01:01:01:01')
        # self.assertEqual(typing1.status, "documented")
        self.assertIsInstance(typing1, Typing)
        pass

    def test_005_A(self):
        #start = time.time()
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb,
                                              port=3307)
        seqann = BioSeqAnn(server=server, dbversion="3200", verbose=True)

        pickle_file1 = "unique_db-feats.pickle"
        pickle_file2 = "feature-service.pickle"
        pickle_gfe2feat = "gfe2feat.pickle"
        pickle_file3 = "gfe2hla.pickle"
        pickle_file4 = "seq2hla.pickle"
        with open(pickle_gfe2feat, 'rb') as handle1:
            gfe_feats = pickle.load(handle1)

        with open(pickle_file1, 'rb') as handle1:
            feats = pickle.load(handle1)

        with open(pickle_file2, 'rb') as handle2:
            cached_feats = pickle.load(handle2)

        with open(pickle_file3, 'rb') as handle3:
            gfe2hla = pickle.load(handle3)

        with open(pickle_file4, 'rb') as handle:
            seq2hla = pickle.load(handle)

        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      load_features=False,
                      verbose=True,
                      features=feats,
                      seq2hla=seq2hla,
                      gfe2hla=gfe2hla,
                      gfe_feats=gfe_feats,
                      cached_features=cached_feats,
                      loci=["HLA-DQB1"])
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/A_fail.fasta", "fasta"))
        typing1 = pygfe.type_from_seq("HLA-DQB1", str(seqs[1].seq), "3.20.0")
        #typing2 = pygfe.type_from_seq("HLA-DRB1", str(seqs[0].seq), "3.31.0")
        #typing2 = pygfe.type_from_seq("HLA-DRB1", str(seqs[0].seq), "3.31.0")
        #end = time.time()
        #time_taken = end - start
        print(typing1)
        #print("=====")
        #print(typing2)
        # self.assertEqual(typing2.hla, 'HLA-A*01:01:01:01')
        # self.assertEqual(typing2.status, "documented")
        #self.assertIsInstance(typing2, Typing)
        # self.assertEqual(typing1.hla, 'HLA-A*01:01:01:01')
        # self.assertEqual(typing1.status, "documented")
        self.assertIsInstance(typing1, Typing)
        pass

    def test_006_align(self):

        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        #if conn():
        server = BioSeqDatabase.open_database(driver="pymysql",
                                              user=biosqluser,
                                              passwd=biosqlpass,
                                              host=biosqlhost,
                                              db=biosqldb,
                                              port=3307)
        seqann = BioSeqAnn(align=True, server=server, dbversion="3310", verbose=True)

        pickle_file1 = "unique_db-feats.pickle"
        pickle_file2 = "feature-service.pickle"
        pickle_gfe2feat = "gfe2feat.pickle"
        pickle_file3 = "gfe2hla.pickle"
        pickle_file4 = "seq2hla.pickle"
        with open(pickle_gfe2feat, 'rb') as handle1:
            gfe_feats = pickle.load(handle1)

        with open(pickle_file1, 'rb') as handle1:
            feats = pickle.load(handle1)

        with open(pickle_file2, 'rb') as handle2:
            cached_feats = pickle.load(handle2)

        with open(pickle_file3, 'rb') as handle3:
            gfe2hla = pickle.load(handle3)

        with open(pickle_file4, 'rb') as handle:
            seq2hla = pickle.load(handle)

        pygfe = pyGFE(graph=graph,
                      seqann=seqann,
                      load_features=False,
                      verbose=True,
                      features=feats,
                      seq2hla=seq2hla,
                      gfe2hla=gfe2hla,
                      gfe_feats=gfe_feats,
                      cached_features=cached_feats,
                      loci=["HLA-A"])
        self.assertIsInstance(pygfe, pyGFE)
        seqs = list(SeqIO.parse(self.data_dir + "/align_tests.fasta", "fasta"))
        typing1 = pygfe.type_from_seq("HLA-A", str(seqs[0].seq), "3.31.0")
        typing2 = pygfe.type_from_seq("HLA-A", str(seqs[1].seq), "3.31.0")
        typing3 = pygfe.type_from_seq("HLA-A", str(seqs[2].seq), "3.31.0")
        typing4 = pygfe.type_from_seq("HLA-A", str(seqs[3].seq), "3.31.0")
        self.assertEqual(typing1.hla, 'HLA-A*02:01:01:12')
        self.assertEqual(typing2.hla, 'HLA-A*02:01:01:12')
        self.assertEqual(typing3.hla, 'HLA-A*02:01:01:12')
        self.assertEqual(typing4.hla, 'HLA-A*02:01:01:12')
        #end = time.time()
        #time_taken = end - start
        #print(typing1)
        #print(typing1.aligned.keys())
        #print(typing1.novel_features)
        #difss = pygfe.hla_seqdiff("HLA-A","3.31.0","HLA-A*01:01:01:01","HLA-A*01:01:01:07")

        #self.assertIsInstance(typing1, Typing)
        pass










#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#    pygfe pyGFE.
#    Copyright (c) 2017 Be The Match operated by National Marrow Donor Program. All Rights Reserved.
#
#    This library is free software; you can redistribute it and/or modify it
#    under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation; either version 3 of the License, or (at
#    your option) any later version.
#
#    This library is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; with out even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
#    License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this library;  if not, write to the Free Software Foundation,
#    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
#
#    > http://www.fsf.org/licensing/licenses/lgpl.html
#    > http://www.opensource.org/licenses/lgpl-license.php
#



"""
test_pygfe
----------------------------------

Tests for `pygfe` module.
"""


import sys
import unittest

from pygfe.gfedb import GfeDB
import os

from py2neo import Graph

neo4jpass = 'gfedb'
if os.getenv("NEO4JPASS"):
    neo4jpass = os.getenv("NEO4JPASS")

neo4juser = 'neo4j'
if os.getenv("NEO4JUSER"):
    neo4juser = os.getenv("NEO4JUSER")

neo4jurl = "http://neo4j.b12x.org:80"
if os.getenv("NEO4JURL"):
    neo4jurl = os.getenv("NEO4JURL")


class TestGfeDB(unittest.TestCase):

    def setUp(self):
        self.data_dir = os.path.dirname(__file__) + "/resources"
        pass

    def tearDown(self):
        pass

    def test_000_pygfe(self):
        self.graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                           bolt=False)
        self.gfedb = GfeDB(self.graph)
        self.assertIsInstance(self.gfedb, GfeDB)
        pass

    # def test_001_pygfe_load(self):
    #     pygfe = pyGFE(verbose=True, load_features=True)
    #     self.assertIsInstance(pygfe, pyGFE)
    #     self.assertGreater(len(pygfe.structures), 1)
    #     self.assertGreater(len(pygfe.all_feats), 1)
    #     self.assertTrue('HLA-A' in pygfe.structures)
    #     self.assertFalse('HLA-Z' in pygfe.structures)
    #     pass









# coding: utf-8

"""

    No descripton provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from __future__ import absolute_import

import os
import sys
import unittest

import swagger_client
from swagger_client.rest import ApiException
from swagger_client.apis.features_api import FeaturesApi


class TestFeaturesApi(unittest.TestCase):
    """ FeaturesApi unit test stubs """

    def setUp(self):
        self.api = swagger_client.apis.features_api.FeaturesApi()

    def tearDown(self):
        pass

    def test_create_feature(self):
        """
        Test case for create_feature

        Create an enumerated sequence feature
        """
        pass

    def test_get_feature_by_path(self):
        """
        Test case for get_feature_by_path

        Retrieve an enumerated sequence feature
        """
        pass

    def test_get_feature_by_query(self):
        """
        Test case for get_feature_by_query

        Retrieve an enumerated sequence feature
        """
        pass

    def test_list_features(self):
        """
        Test case for list_features

        List the enumerated sequence features at a locus
        """
        pass

    def test_list_features_0(self):
        """
        Test case for list_features_0

        List the enumerated sequence features matching a term at a locus
        """
        pass

    def test_list_features_1(self):
        """
        Test case for list_features_1

        List the enumerated sequence features matching a term and rank at a locus
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""

    No descripton provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)

    OpenAPI spec version: 1.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""

from pprint import pformat
from six import iteritems
from typing import List
from pygfe.feature_client.models.feature import Feature


class Sequence(object):
    """
    NOTE: This class is auto generated by the swagger code generator program.
    Do not edit the class manually.
    """
    def __init__(self, sequence=str, structure=List[Feature], hash_code=None):
        """
        Feature - a model defined in Swagger

        :param dict swaggerTypes: The key is attribute name
                                  and the value is attribute type.
        :param dict attributeMap: The key is attribute name
                                  and the value is json key in definition.
        """
        self.swagger_types = {
            'structure': 'List[Feature]',
            'sequence': 'str',
            'hash_code': 'int'
        }

        self.attribute_map = {
            'structure': 'structure',
            'sequence': 'sequence',
            'hash_code': 'hash_code'
        }

        self._structure = structure
        self._sequence = sequence
        self._hash_code = hash_code

    @property
    def sequence(self):
        """
        Gets the sequence of this Sequence.


        :return: The sequence of this Sequence.
        :rtype: str
        """
        return self._structure

    @sequence.setter
    def sequence(self, sequence):
        """
        Sets the sequence of this Sequence.


        :param sequence: The sequence of this Sequence.
        :type: str
        """

        self._sequence = sequence

    @property
    def structure(self):
        """
        Gets the structure of this Sequence.


        :return: The structure of this Sequence.
        :rtype: str
        """
        return self._structure

    @structure.setter
    def structure(self, structure):
        """
        Sets the structure of this Feature.


        :param structure: The structure of this Feature.
        :type: str
        """

        self._structure = structure

    @property
    def hash_code(self):
        """
        Gets the hash_code of this Feature.


        :return: The hash_code of this Feature.
        :rtype: int
        """
        return self._hash_code

    @hash_code.setter
    def hash_code(self, hash_code):
        """
        Sets the hash_code of this Feature.


        :param hash_code: The hash_code of this Feature.
        :type: int
        """

        self._hash_code = hash_code

    def to_dict(self):
        """
        Returns the model properties as a dict
        """
        result = {}

        for attr, _ in iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """
        Returns the string representation of the model
        """
        return pformat(self.to_dict())

    def __repr__(self):
        """
        For `print` and `pprint`
        """
        return self.to_str()

    def __eq__(self, other):
        """
        Returns true if both objects are equal
        """
        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """
        Returns true if both objects are not equal
        """
        return not self == other



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from pygfe.models.base_model_ import Model
from seqann import util


class Feature(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, accession: int=None, rank: int=None, sequence: str=None, term: str=None):  # noqa: E501
        """Feature - a model defined in Swagger

        :param accession: The accession of this Feature.  # noqa: E501
        :type accession: int
        :param rank: The rank of this Feature.  # noqa: E501
        :type rank: int
        :param sequence: The sequence of this Feature.  # noqa: E501
        :type sequence: str
        :param term: The term of this Feature.  # noqa: E501
        :type term: str
        """
        self.swagger_types = {
            'accession': int,
            'rank': int,
            'sequence': str,
            'term': str
        }

        self.attribute_map = {
            'accession': 'accession',
            'rank': 'rank',
            'sequence': 'sequence',
            'term': 'term'
        }

        self._accession = accession
        self._rank = rank
        self._sequence = sequence
        self._term = term

    @classmethod
    def from_dict(cls, dikt) -> 'Feature':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Feature of this Feature.  # noqa: E501
        :rtype: Feature
        """
        return util.deserialize_model(dikt, cls)

    @property
    def accession(self) -> int:
        """Gets the accession of this Feature.


        :return: The accession of this Feature.
        :rtype: int
        """
        return self._accession

    @accession.setter
    def accession(self, accession: int):
        """Sets the accession of this Feature.


        :param accession: The accession of this Feature.
        :type accession: int
        """

        self._accession = accession

    @property
    def rank(self) -> int:
        """Gets the rank of this Feature.


        :return: The rank of this Feature.
        :rtype: int
        """
        return self._rank

    @rank.setter
    def rank(self, rank: int):
        """Sets the rank of this Feature.


        :param rank: The rank of this Feature.
        :type rank: int
        """

        self._rank = rank

    @property
    def sequence(self) -> str:
        """Gets the sequence of this Feature.


        :return: The sequence of this Feature.
        :rtype: str
        """
        return self._sequence

    @sequence.setter
    def sequence(self, sequence: str):
        """Sets the sequence of this Feature.


        :param sequence: The sequence of this Feature.
        :type sequence: str
        """

        self._sequence = sequence

    @property
    def term(self) -> str:
        """Gets the term of this Feature.


        :return: The term of this Feature.
        :rtype: str
        """
        return self._term

    @term.setter
    def term(self, term: str):
        """Sets the term of this Feature.


        :param term: The term of this Feature.
        :type term: str
        """

        self._term = term



# coding: utf-8

# flake8: noqa
from __future__ import absolute_import
# import models into model package
from pygfe.models.error import Error
from pygfe.models.feature import Feature
from pygfe.models.seqdiff import Seqdiff
from pygfe.models.typing import Typing



# coding: utf-8

from __future__ import absolute_import
from datetime import date, datetime  # noqa: F401

from typing import List, Dict  # noqa: F401

from pygfe.models.base_model_ import Model
from pygfe.models.feature import Feature  # noqa: F401,E501
from pygfe.models.seqdiff import Seqdiff  # noqa: F401,E501
from seqann import util


class Typing(Model):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    def __init__(self, seqdiff: List[Seqdiff]=None, protdiff: List[Seqdiff]=None, novel_features: List[Feature]=None, features: List[Feature]=None, gfe: str=None, hla: str=None, closest_gfe: str=None, full_gene_accession: int=None, differences: int=None, status: str=None, seqann_version: str=None, pygfe_version: str=None, gfedb_version: str=None, imgtdb_version: str=None):  # noqa: E501
        """Typing - a model defined in Swagger

        :param seqdiff: The seqdiff of this Typing.  # noqa: E501
        :type seqdiff: List[Seqdiff]
        :param protdiff: The protdiff of this Typing.  # noqa: E501
        :type protdiff: List[Seqdiff]
        :param features: The features of this Typing.  # noqa: E501
        :type features: List[Feature]
        :param gfe: The gfe of this Typing.  # noqa: E501
        :type gfe: str
        :param hla: The hla of this Typing.  # noqa: E501
        :type hla: str
        :param closest_gfe: The closest_gfe of this Typing.  # noqa: E501
        :type closest_gfe: str
        :param full_gene_accession: The full_gene_accession of this Typing.  # noqa: E501
        :type full_gene_accession: int
        :param differences: The differences of this Typing.  # noqa: E501
        :type differences: int
        :param status: The status of this Typing.  # noqa: E501
        :type status: str
        :param pygfe_version: The pygfe_version of this Typing.  # noqa: E501
        :type pygfe_version: str
        :param gfedb_version: The gfedb_version of this Typing.  # noqa: E501
        :type gfedb_version: str
        :param imgtdb_version: The imgtdb_version of this Typing.  # noqa: E501
        :type imgtdb_version: str
        """
        self.swagger_types = {
            'seqdiff': List[Seqdiff],
            'protdiff': List[Seqdiff],
            'features': List[Feature],
            'novel_features': List[Feature],
            'gfe': str,
            'hla': str,
            'closest_gfe': str,
            'full_gene_accession': int,
            'differences': int,
            'status': str,
            'pygfe_version': str,
            'gfedb_version': str,
            'imgtdb_version': str,
            'seqann_version': str
        }

        self.attribute_map = {
            'seqdiff': 'seqdiff',
            'protdiff': 'protdiff',
            'features': 'features',
            'novel_features': 'novel_features',
            'gfe': 'gfe',
            'hla': 'hla',
            'closest_gfe': 'closest_gfe',
            'full_gene_accession': 'full_gene_accession',
            'differences': 'differences',
            'status': 'status',
            'pygfe_version': 'pygfe_version',
            'gfedb_version': 'gfedb_version',
            'imgtdb_version': 'imgtdb_version',
            'seqann_version': 'seqann_version'
        }

        self._seqann_version = seqann_version
        self._seqdiff = seqdiff
        self._protdiff = protdiff
        self._features = features
        self._novel_features = novel_features
        self._gfe = gfe
        self._hla = hla
        self._closest_gfe = closest_gfe
        self._full_gene_accession = full_gene_accession
        self._differences = differences
        self._status = status
        self._pygfe_version = pygfe_version
        self._gfedb_version = gfedb_version
        self._imgtdb_version = imgtdb_version

    @classmethod
    def from_dict(cls, dikt) -> 'Typing':
        """Returns the dict as a model

        :param dikt: A dict.
        :type: dict
        :return: The Typing of this Typing.  # noqa: E501
        :rtype: Typing
        """
        return util.deserialize_model(dikt, cls)

    @property
    def seqdiff(self) -> List[Seqdiff]:
        """Gets the seqdiff of this Typing.


        :return: The seqdiff of this Typing.
        :rtype: List[Seqdiff]
        """
        return self._seqdiff

    @seqdiff.setter
    def seqdiff(self, seqdiff: List[Seqdiff]):
        """Sets the seqdiff of this Typing.


        :param seqdiff: The seqdiff of this Typing.
        :type seqdiff: List[Seqdiff]
        """

        self._seqdiff = seqdiff

    @property
    def protdiff(self) -> List[Seqdiff]:
        """Gets the protdiff of this Typing.


        :return: The protdiff of this Typing.
        :rtype: List[Seqdiff]
        """
        return self._protdiff

    @protdiff.setter
    def protdiff(self, protdiff: List[Seqdiff]):
        """Sets the protdiff of this Typing.


        :param protdiff: The protdiff of this Typing.
        :type protdiff: List[Seqdiff]
        """

        self._protdiff = protdiff

    @property
    def features(self) -> List[Feature]:
        """Gets the features of this Typing.


        :return: The features of this Typing.
        :rtype: List[Feature]
        """
        return self._features

    @features.setter
    def features(self, features: List[Feature]):
        """Sets the features of this Typing.


        :param features: The features of this Typing.
        :type features: List[Feature]
        """

        self._features = features

    @property
    def novel_features(self) -> List[Feature]:
        """Gets the novel_features of this Typing.


        :return: The novel_features of this Typing.
        :rtype: List[Feature]
        """
        return self._novel_features

    @novel_features.setter
    def novel_features(self, novel_features: List[Feature]):
        """Sets the novel_features of this Typing.


        :param novel_features: The novel_features of this Typing.
        :type novel_features: List[Feature]
        """

        self._novel_features = novel_features

    @property
    def gfe(self) -> str:
        """Gets the gfe of this Typing.


        :return: The gfe of this Typing.
        :rtype: str
        """
        return self._gfe

    @gfe.setter
    def gfe(self, gfe: str):
        """Sets the gfe of this Typing.


        :param gfe: The gfe of this Typing.
        :type gfe: str
        """

        self._gfe = gfe

    @property
    def hla(self) -> str:
        """Gets the hla of this Typing.


        :return: The hla of this Typing.
        :rtype: str
        """
        return self._hla

    @hla.setter
    def hla(self, hla: str):
        """Sets the hla of this Typing.


        :param hla: The hla of this Typing.
        :type hla: str
        """

        self._hla = hla

    @property
    def closest_gfe(self) -> str:
        """Gets the closest_gfe of this Typing.


        :return: The closest_gfe of this Typing.
        :rtype: str
        """
        return self._closest_gfe

    @closest_gfe.setter
    def closest_gfe(self, closest_gfe: str):
        """Sets the closest_gfe of this Typing.


        :param closest_gfe: The closest_gfe of this Typing.
        :type closest_gfe: str
        """

        self._closest_gfe = closest_gfe

    @property
    def full_gene_accession(self) -> int:
        """Gets the full_gene_accession of this Typing.


        :return: The full_gene_accession of this Typing.
        :rtype: int
        """
        return self._full_gene_accession

    @full_gene_accession.setter
    def full_gene_accession(self, full_gene_accession: int):
        """Sets the full_gene_accession of this Typing.


        :param full_gene_accession: The full_gene_accession of this Typing.
        :type full_gene_accession: int
        """

        self._full_gene_accession = full_gene_accession

    @property
    def differences(self) -> int:
        """Gets the differences of this Typing.


        :return: The differences of this Typing.
        :rtype: int
        """
        return self._differences

    @differences.setter
    def differences(self, differences: int):
        """Sets the differences of this Typing.


        :param differences: The differences of this Typing.
        :type differences: int
        """

        self._differences = differences

    @property
    def status(self) -> str:
        """Gets the status of this Typing.


        :return: The status of this Typing.
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status: str):
        """Sets the status of this Typing.


        :param status: The status of this Typing.
        :type status: str
        """

        self._status = status

    @property
    def pygfe_version(self) -> str:
        """Gets the pygfe_version of this Typing.


        :return: The pygfe_version of this Typing.
        :rtype: str
        """
        return self._pygfe_version

    @pygfe_version.setter
    def pygfe_version(self, pygfe_version: str):
        """Sets the pygfe_version of this Typing.


        :param pygfe_version: The pygfe_version of this Typing.
        :type pygfe_version: str
        """

        self._pygfe_version = pygfe_version

    @property
    def gfedb_version(self) -> str:
        """Gets the gfedb_version of this Typing.


        :return: The gfedb_version of this Typing.
        :rtype: str
        """
        return self._gfedb_version

    @gfedb_version.setter
    def gfedb_version(self, gfedb_version: str):
        """Sets the gfedb_version of this Typing.


        :param gfedb_version: The gfedb_version of this Typing.
        :type gfedb_version: str
        """

        self._gfedb_version = gfedb_version

    @property
    def seqann_version(self) -> str:
        """Gets the seqann_version of this Typing.


        :return: The seqann_version of this Typing.
        :rtype: str
        """
        return self._seqann_version

    @seqann_version.setter
    def seqann_version(self, seqann_version: str):
        """Sets the seqann_version of this Typing.


        :param seqann_version: The seqann_version of this Typing.
        :type seqann_version: str
        """

        self._seqann_version = seqann_version

    @property
    def imgtdb_version(self) -> str:
        """Gets the imgtdb_version of this Typing.


        :return: The imgtdb_version of this Typing.
        :rtype: str
        """
        return self._imgtdb_version

    @imgtdb_version.setter
    def imgtdb_version(self, imgtdb_version: str):
        """Sets the imgtdb_version of this Typing.


        :param imgtdb_version: The imgtdb_version of this Typing.
        :type imgtdb_version: str
        """

        self._imgtdb_version = imgtdb_version



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#    pygfe pyGFE.
#    Copyright (c) 2017 Be The Match operated by National Marrow Donor Program. All Rights Reserved.
#
#    This library is free software; you can redistribute it and/or modify it
#    under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation; either version 3 of the License, or (at
#    your option) any later version.
#
#    This library is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; with out even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
#    License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this library;  if not, write to the Free Software Foundation,
#    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
#
#    > http://www.fsf.org/licensing/licenses/lgpl.html
#    > http://www.opensource.org/licenses/lgpl-license.php
#



"""
test_pygfe
----------------------------------

Tests for `pygfe` module.
"""


import sys
import unittest

import os
from Bio import SeqIO
from pygfe.act import ACT
from py2neo import Graph
from pygfe.gfedb import GfeDB
from pygfe.gfe import GFE
from pygfe.models.allele_call import AlleleCall

from seqann import BioSeqAnn
from BioSQL import BioSeqDatabase

import pymysql


neo4jpass = 'gfedb'
if os.getenv("NEO4JPASS"):
    neo4jpass = os.getenv("NEO4JPASS")

neo4juser = 'neo4j'
if os.getenv("NEO4JUSER"):
    neo4juser = os.getenv("NEO4JUSER")

neo4jurl = "http://neo4j.b12x.org:80"
if os.getenv("NEO4JURL"):
    neo4jurl = os.getenv("NEO4JURL")

biosqlpass = ''
if os.getenv("BIOSQLPASS"):
    biosqlpass = os.getenv("BIOSQLPASS")

biosqluser = ''
if os.getenv("BIOSQLUSER"):
    biosqluser = os.getenv("BIOSQLUSER")

biosqlhost = "localhost"
if os.getenv("BIOSQLHOST"):
    biosqlhost = os.getenv("BIOSQLHOST")

biosqldb = "bioseqdb"
if os.getenv("BIOSQLDB"):
    biosqldb = os.getenv("BIOSQLDB")

biosqlport = 3306
if os.getenv("BIOSQLPORT"):
    biosqlport = os.getenv("BIOSQLPORT")


def conn():
    try:
        # print(biosqlpass, biosqluser, biosqlhost, biosqldb, biosqlport, sep="\t")
        conn = pymysql.connect(host=biosqlhost,
                               port=biosqlport, user=biosqluser,
                               passwd=biosqlpass, db=biosqldb)
        conn.close()
        return True
    except Exception as e:
        print("Exception while checking MYSQL Connection:" + str(e))
        return False


class TestAct(unittest.TestCase):

    def setUp(self):
        self.data_dir = os.path.dirname(__file__) + "/resources"

        pass

    def tearDown(self):
        pass

    # def test_000_act(self):
    #     graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
    #                   bolt=False)
    #     if conn():
    #         server = BioSeqDatabase.open_database(driver="pymysql",
    #                                               user=biosqluser,
    #                                               passwd=biosqlpass,
    #                                               host=biosqlhost,
    #                                               db=biosqldb)
    #         print("Server found!", file=sys.stderr)
    #         seqann = BioSeqAnn(server=server)
    #     else:
    #         print("No Server found!", file=sys.stderr)
    #         seqann = BioSeqAnn()
    #     act = ACT(gfedb=GfeDB(graph),
    #               seqann=seqann,
    #               gfe=GFE())

    #     self.assertIsInstance(act, ACT)
    #     pass

    # def test_001_type_sequence(self):
    #     act = ACT(self.graph)
    #     seqs = list(SeqIO.parse(self.data_dir + "/known_A.fasta", "fasta"))
    #     typing = act.type_hla("HLA-A", str(seqs[0].seq), '')
    #     print(type(typing))
    #     self.assertIsInstance(act, ACT)
    #     pass

    # def test_002_type_unknown(self):
    #     seqs = list(SeqIO.parse(self.data_dir + "/unknown_A.fasta", "fasta"))
    #     typing = self.act.type_from_seq("HLA-A", str(seqs[0].seq))
    #     self.assertIsInstance(self.act, ACT)
    #     self.assertIsInstance(typing, AlleleCall)
    #     pass

    def test_002_type_unknown(self):
        #test_id = act.get_alleleid("A*01:01:01:01")
        graph = Graph(neo4jurl, user=neo4juser, password=neo4jpass,
                      bolt=False)
        if conn():
            server = BioSeqDatabase.open_database(driver="pymysql",
                                                  user=biosqluser,
                                                  passwd=biosqlpass,
                                                  host=biosqlhost,
                                                  db=biosqldb)
            print("Server found!", file=sys.stderr)
            seqann = BioSeqAnn(server=server)
        else:
            print("No Server found!", file=sys.stderr)
            seqann = BioSeqAnn()
        act = ACT(gfedb=GfeDB(graph),
                  seqann=seqann,
                  gfe=GFE())

        seqs = list(SeqIO.parse(self.data_dir + "/unknown_A.fasta", "fasta"))
        typing = act.type_from_seq("HLA-A", str(seqs[0].seq))
        print(typing)
        print("TEST 2")
        print(type(typing))
        self.assertIsInstance(act, ACT)
        # self.assertGreater(len(pygfe.structures), 1)
        # self.assertTrue('HLA-A' in pygfe.structures)
        # self.assertFalse('HLA-Z' in pygfe.structures)
        pass

    # def test_003_pygfe(self):
    #     act = ACT(self.graph)
    #     #test_id = act.get_alleleid("A*01:01:01:01")
    #     seqs = list(SeqIO.parse(self.data_dir + "/test_seq.fasta", "fasta"))
    #     lookups = act.type_hla("HLA-A", str(seqs[0].seq), '')
    #     #print(lookups)
    #     #lookups = act.sequence_lookup("HLA-A", str(seqs[0].seq))
    #     self.assertIsInstance(act, ACT)
    #     # self.assertGreater(len(pygfe.structures), 1)
    #     # self.assertTrue('HLA-A' in pygfe.structures)
    #     # self.assertFalse('HLA-Z' in pygfe.structures)
    #     pass
    # def test_001_pygfe_load(self):
    #     pygfe = pyGFE(verbose=True, load_features=True)
    #     self.assertIsInstance(pygfe, pyGFE)
    #     self.assertGreater(len(pygfe.structures), 1)
    #     self.assertGreater(len(pygfe.all_feats), 1)
    #     self.assertTrue('HLA-A' in pygfe.structures)
    #     self.assertFalse('HLA-Z' in pygfe.structures)
    #     pass









#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#    pygfe pyGFE.
#    Copyright (c) 2017 Be The Match operated by National Marrow Donor Program. All Rights Reserved.
#
#    This library is free software; you can redistribute it and/or modify it
#    under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation; either version 3 of the License, or (at
#    your option) any later version.
#
#    This library is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; with out even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
#    License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this library;  if not, write to the Free Software Foundation,
#    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
#
#    > http://www.fsf.org/licensing/licenses/lgpl.html
#    > http://www.opensource.org/licenses/lgpl-license.php
#



"""
test_pygfe
----------------------------------

Tests for `pygfe` module.
"""


import sys
import unittest

from pygfe.gfe import GFE
import os


class TestPygfe(unittest.TestCase):

    def setUp(self):
        self.data_dir = os.path.dirname(__file__) + "/resources"
        pass

    def tearDown(self):
        pass

    def test_000_pygfe(self):
        pygfe = GFE()
        self.assertIsInstance(pygfe, GFE)
        self.assertGreater(len(pygfe.structures), 1)
        self.assertTrue('HLA-A' in pygfe.structures)
        self.assertFalse('HLA-Z' in pygfe.structures)
        pass

    def test_001_pygfe_load(self):
        pygfe = GFE(verbose=True, load_features=True)
        self.assertIsInstance(pygfe, GFE)
        self.assertGreater(len(pygfe.structures), 1)
        self.assertGreater(len(pygfe.all_feats), 1)
        self.assertTrue('HLA-A' in pygfe.structures)
        self.assertFalse('HLA-Z' in pygfe.structures)
        pass









'''
Created on Feb 8, 2017

@author: mhalagan
'''

import pygfe
from pygfe.cypher import sequence_search, all_db_imgt
from pygfe.cypher import similar_gfe_classI
from pygfe.cypher import similar_gfe_classII
from pygfe.cypher import similar_kir
from pygfe.cypher import get_features
from pygfe.cypher import search_feature
from pygfe.models.feature import Feature
from pygfe.models.typing import Typing
from pygfe.models.error import Error
from pygfe.models.seqdiff import Seqdiff
from pygfe.cypher import all_gfe2hla
from pygfe.cypher import all_seq2hla
from pygfe.cypher import all_gfe2feats
from pygfe.util import get_structures

import seqann
from seqann.blast_cmd import get_locus
from seqann.util import checkseq
from seqann.util import isutr
from seqann.util import is_classII
from seqann.util import is_kir
from seqann import BioSeqAnn

from Bio.Seq import Seq
from Bio.Alphabet import IUPAC
from Bio.SeqRecord import SeqRecord

from pandas import DataFrame

from py2neo import Graph

from typing import List
from typing import Dict
from typing import Any

import re
import time
import logging

flatten = lambda l: [item for sublist in l for item in sublist]
is_gfe = lambda x: True if re.search("\d+-\d+-\d+", x) else False
lc = lambda x: x.lower() if not re.search("UTR", x) else x.lower().replace("utr", "UTR")
is_classI = lambda x: True if re.search("HLA-\Dw", x) else False

# Dict of seqann objects

# ** May want to eventually create a GFE service

# TODO: have option for not returning features

# seqann = BioSeqAnn(dbversion="3310")
# gfe = pyGFE(seann={"3310": seqann})
# gfe = pyGFE(seqann=seqann})
# gfe = pyGFE(seqann=[seqann1, seqann2]})


class pyGFE(object):
    '''
    classdocs
    '''
    def __init__(self, url="http://feature.nmdp-bioinformatics.org",
                 loci=['HLA-A', 'HLA-B', 'HLA-C',
                       'HLA-DRB1', 'HLA-DQB1', 'HLA-DRB4',
                       'HLA-DRB5', 'HLA-DPB1', 'HLA-DPA1',
                       'HLA-DQA1', 'HLA-DRB3'],
                 graph: Graph=None,
                 seqann: Any={},
                 features: Dict=None,
                 verbose: bool=False,
                 kir: bool=False,
                 pid: str="NA",
                 gfe2hla: Dict=None,
                 gfe_feats: DataFrame=None,
                 seq2hla: DataFrame=None,
                 load_gfe2hla: bool=False,
                 load_seq2hla: bool=False,
                 load_gfe2feat: bool=False,
                 verbosity=1):
        '''
        Constructor
        '''
        # TODO: Add catch if seqann or graph aren't defined
        self.kir = kir
        self.graph = graph
        self.logger = logging.getLogger("Logger." + __name__)
        if pid:
            self.logname = "ID {:<10} - ".format(str(pid))
        else:
            self.logname = ''

        if not isinstance(seqann, Dict) and seqann:
            if isinstance(seqann, BioSeqAnn):
                self.seqann = {seqann.refdata.dbversion: seqann}
            elif(isinstance(seqann, List)):
                self.seqann = {}
                for ann in seqann:
                    self.seqann.update({ann.refdata.dbversion: ann})
            else:
                raise SeqAnnException(inputtype=type(seqann),
                                      reason="Can't initalize seqann")
        else:
            self.seqann = seqann

        self.features = features
        self.gfe2hla = gfe2hla
        self.seq2hla = seq2hla
        self.gfe_feats = gfe_feats
        self.verbose = verbose
        self.structures = get_structures()

        # ISSUE: gfe_feats & seq2hla need to be loaded together
        #
        if load_gfe2feat:
            self.gfe_feats = self.graph.run(all_gfe2feats()).to_data_frame()
            self.gfe_feats['DBV'] = self.gfe_feats['DB'].apply(lambda db: "".join(db.split(".")))
            self.gfe_feats['DB'] = self.gfe_feats['DBV']
            self.gfe_feats = self.gfe_feats.drop(['DBV'], axis=1)

        if load_seq2hla:
            self.seq2hla = self.graph.run(all_seq2hla()).to_data_frame()
            self.seq2hla['DBV'] = self.seq2hla['DB'].apply(lambda db: "".join(db.split(".")))
            self.seq2hla['DB'] = self.seq2hla['DBV']
            self.seq2hla = self.seq2hla.drop(['DBV'], axis=1)

        if load_gfe2hla:
            tmp_gfe = {}
            gfehla_df = self.graph.run(all_gfe2hla()).to_data_frame()
            for loc in gfehla_df['LOC'].unique().tolist():
                if re.search("HLA-\D$", loc):
                    loc_df = gfehla_df.loc[gfehla_df['LOC'] == loc]
                    loc1 = self.structures[loc]['exon-2']
                    loc2 = self.structures[loc]['exon-3']
                    loc_df['EXON23'] = loc_df['GFE'].apply(lambda gfe: "-".join([gfe.split("-")[loc1],gfe.split("-")[loc2]]))
                    tmp_gfe.update({loc: loc_df})

                if is_classII(loc):
                    loc_df = gfehla_df.loc[gfehla_df['LOC'] == loc]
                    loc1 = self.structures[loc]['exon-2']
                    loc_df['EXON2'] = loc_df['GFE'].apply(lambda gfe:gfe.split("-")[loc1])
                    tmp_gfe.update({loc: loc_df})
            self.gfe2hla = tmp_gfe

    # TODO: Make this come from the
    #       pygfe cache instead of making
    #       a bunch of calls to the GFE DB
    def unique_features(self, features, locus, imgtdb_version):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.
        :return: GFEobject.
        """
        unique = []
        if self.features:
            for feat in features:
                rank = feat.rank if not isutr(feat.term) else 1
                feat_str = ":".join([imgtdb_version,
                                     locus, str(rank),
                                     feat.term.upper(), feat.sequence])
                if not feat_str in self.features:
                    unique.append(feat)
        else:
            for feat in features:
                feat_q = search_feature(feat.term.upper(),
                                        feat.rank,
                                        feat.sequence)

                seq_features = self.graph.run(feat_q).to_data_frame()
                if seq_features.empty:
                    unique.append(feat)
        return unique

    # TODO: have parameter for filling in
    #       seqann parameters if it's a new
    #       dbversion
    def type_from_seq(self, locus: str=None,
                      sequence: str=None,
                      imgtdb_version: str="3.31.0",
                      nseqs: int=20,
                      alignseqs: int=10,
                      skip: List=[]):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """

        # TODO: Add full gene accession
        # TODO: reformt dbversion if missing .
        ac_object = Typing()
        ac_object.imgtdb_version = "".join(imgtdb_version.split("."))
        ac_object.pygfe_version = pygfe.__version__
        ac_object.seqann_version = seqann.__version__
        ac_object.gfedb_version = '0.0.2'

        # If sequence is now a biopython
        # sequence record convert it to one
        if isinstance(sequence, Seq):
            sequence = str(sequence)
        elif(isinstance(sequence, SeqRecord)):
            sequence = str(sequence.seq)

        if not ac_object.imgtdb_version in self.seqann:
            self.seqann.update({ac_object.imgtdb_version:
                                BioSeqAnn(dbversion=ac_object.imgtdb_version,
                                          #store_features=self.store_features,
                                          load_features=self.load_features,
                                          cached_features=self.cached_features)})

        # If sequence contains any characters
        # other than ATCG then the GFE notation
        # can not be created
        valid_seq = checkseq(sequence)

        if self.verbose and not valid_seq:
            self.logger.warning(self.logname + " Sequence alphabet "
                                + "contains non DNA")
            self.logger.warning(self.logname
                                + " No GFE string will be generated")
            raise Exception("Input sequence was not valid! {}".format(sequence))

        # Check it the locus exists
        if not locus:
            if self.verbose:
                self.logger.info(self.logname + " No locus provided! ")

            # Guessing locus with blastn
            locus = get_locus(sequence,
                              kir=self.kir,
                              refdata=self.seqann[ac_object.imgtdb_version].refdata)

            if locus and self.verbose:
                self.logger.info(self.logname + " Locus prediction = " + locus)

            if not locus:
                if self.verbose:
                    self.logger.error(self.logname
                                      + " Locus could not be determined!")
                # TODO: Raise exception
                raise Exception("Locus could not be determined! {}".format(sequence))

        sequence = sequence.upper()
        sequence_typing = self.sequence_lookup(locus,
                                               sequence,
                                               ac_object.imgtdb_version)
        if sequence_typing:
            ac_object.status = "documented"
            ac_object.hla = sequence_typing[0]
            ac_object.gfe = sequence_typing[1]
            ac_object.closest_gfe = sequence_typing[1]
            ac_object.features = sequence_typing[2]

            if self.verbose:
                self.logger.info(self.logname + locus + " sequence documented for " + imgtdb_version + " | " + ac_object.gfe + " = " + ac_object.hla)

            return ac_object
        else:
            # time GFE creation
            time_start = time.time()
            gfe_o = self.gfe_create(locus, sequence, ac_object.imgtdb_version)
            if not 'annotation' in gfe_o:
                self.logger.error(self.logname + "Failed to create annotation!!")
                error = Error("Failed to create annotation!!", ac_object.pygfe_version, ac_object.gfedb_version, imgtdb_version)
                return error

            if self.verbose:
                time_taken = int(time.time() - time_start)
                self.logger.info(self.logname + " gfe_create time for " + locus + " "
                                 + imgtdb_version + " = " + str(time_taken) + " minutes")
            annotation = gfe_o['annotation']
            ac_object.gfe = gfe_o['gfe']
            ac_object.features = [Feature(accession=f.accession,
                                          rank=f.rank,
                                          sequence=f.sequence,
                                          term=f.term) for f in gfe_o['structure']]
            novel_features = self.unique_features(ac_object.features,
                                                  locus,
                                                  ac_object.imgtdb_version)
            if(len(novel_features) != 0):
                if self.verbose:
                    self.logger.info(self.logname + " # novel features = "
                                     + str(len(novel_features)))
                ac_object.novel_features = novel_features
                ac_object.status = "novel"
            else:
                self.logger.info(self.logname + " novel combination")
                ac_object.status = "novel_combination"

            similar_results = self.find_similar(ac_object.gfe,
                                                ac_object.features,
                                                imgtdb_version)
            if similar_results:
                ac_object.hla = similar_results[0]
                ac_object.closest_gfe = similar_results[1]
                if self.seqann[ac_object.imgtdb_version].align:
                    if self.verbose:
                        self.logger.info(self.logname + " finding sequence differences")
                    ac_object.seqdiff = self.diff_seq(similar_results[0],
                                                      annotation,
                                                      imgtdb_version)
                    ac_object.differences = len(ac_object.seqdiff)
            else:
                ac_object.hla = "NA"
                ac_object.closest_gfe = "NA"
                if self.verbose:
                    self.logger.warn(self.logname + " No allele call made!")
            return ac_object

    def sequence_lookup(self, locus, sequence, imgtdb_version):
        """
        Looks up sequence from

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """
        # TODO: just initalize seq2hla as DataFrame
        if isinstance(self.seq2hla, DataFrame) and not self.seq2hla.empty:
            df = self.seq2hla[(self.seq2hla['DB'] == imgtdb_version) & (self.seq2hla['LOC'] == locus) & (self.seq2hla['SEQ'] == sequence)]
            if not df.empty:
                hla = df['HLA'].tolist()[0]
                gfe = df['GFE'].tolist()[0]
                features = []
                feats = self.gfe_feats[(self.gfe_feats['GFE'] == gfe) &
                                          (self.gfe_feats['DB'] == imgtdb_version)]['FEATS'].tolist()[0]
                if feats and len(feats) > 0:
                    for feat in feats:
                        feature = Feature(accession=feat['accession'],
                                          rank=feat['rank'],
                                          sequence=feat['sequence'],
                                          term=lc(feat['term']))
                        features.append(feature)
                else:
                    seq_features = self.graph.run(get_features(gfe)).to_data_frame()
                    for i in range(0, len(seq_features['term'])):
                        feature = Feature(accession=seq_features['accession'][i],
                                          rank=seq_features['rank'][i],
                                          sequence=seq_features['sequence'][i],
                                          term=lc(seq_features['term'][i]))
                        features.append(feature)
                return [hla, gfe, features]
        else:
            lookup_query = sequence_search(locus, sequence)
            sequence_data = self.graph.run(lookup_query).to_data_frame()
            if not sequence_data.empty:
                features = list()
                gfe = list(set([x for x in sequence_data["GFE"]]))
                hla = list(set([x for x in sequence_data["HLA"]]))
                seq_features = self.graph.run(get_features(gfe[0])).to_data_frame()
                for i in range(0, len(seq_features['term'])):
                    feature = Feature(accession=seq_features['accession'][i],
                                      rank=seq_features['rank'][i],
                                      sequence=seq_features['sequence'][i],
                                      term=lc(seq_features['term'][i]))
                    features.append(feature)
                return [hla[0], gfe[0], features]
            else:
                return

    def gfe_create(self, locus, sequence, imgtdb_version):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: Dict.
        """
        seq_rec = SeqRecord(Seq(sequence, IUPAC.unambiguous_dna), id="GFE")
        annotation = self.seqann[imgtdb_version].annotate(seq_rec, locus)
        if not hasattr(annotation, 'annotation') or not hasattr(annotation, 'gfe'):
            return {}
        else:
            return {'gfe': annotation.gfe, 'structure': annotation.structure, 'annotation': annotation}

    def find_similar(self, gfe, features, imgtdb_version):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """
        if is_classI(gfe):
            gfe_dict = self.breakup_gfe(gfe)
            [locus, feature_accessions] = gfe.split("w")
            if self.gfe2hla:
                exon23 = "-".join([gfe_dict["EXON-2"], gfe_dict["EXON-3"]])
                df = self.gfe2hla[locus][(self.gfe2hla[locus]['EXON23'] == exon23) & (self.gfe2hla[locus]['DB'] == imgtdb_version)][['GFE', 'HLA']]\
                         .reset_index()
                return self.create_typing(df, gfe, features)
            else:
                cypher = similar_gfe_classI(gfe, gfe_dict["EXON-2"],
                                            gfe_dict["EXON-3"],
                                            imgtdb_version)
                similar_data = self.graph.run(cypher).to_data_frame()
                return self.create_typing(similar_data, gfe, features)
        elif is_classII(gfe):
            gfe_dict = self.breakup_gfe(gfe)
            [locus, feature_accessions] = gfe.split("w")
            if self.gfe2hla:
                exon2 = gfe_dict["EXON-2"]
                df = self.gfe2hla[locus][(self.gfe2hla[locus]['EXON2'] == exon2) & (self.gfe2hla[locus]['DB'] == imgtdb_version)][['GFE', 'HLA']]\
                         .reset_index()
                return self.create_typing(df, gfe, features)
            else:
                cypher = similar_gfe_classII(gfe, gfe_dict["EXON-2"],
                                             imgtdb_version)
                similar_data = self.graph.run(cypher).to_data_frame()
                return self.create_typing(similar_data, gfe, features)
        elif is_kir(gfe):
            return self.find_gfe_kir(gfe, features)
        else:
            return

    def rename_feat(self, f):
        if isutr(f):
            return f
        else:
            return "_".join(f.split("-"))

    def diff_seq(self, ref_allele, annotation, imgtdb_version):
        # TODO: get neo4j seq array back as python array
        #       and then do the comparison in python.
        locus = ref_allele.split("*")[0]
        dbv = "".join(imgtdb_version.split("."))
        feat_order = list(self.seqann[dbv].refdata.struct_order[locus].keys())
        feat_order.sort()
        aligned_seq = list("".join([annotation.aligned[self.rename_feat(self.seqann[dbv].refdata.struct_order[locus][i])] for i in feat_order]))
        seq_l = ",".join(["".join(["\"", s, "\""]) for s in aligned_seq])
        query = "MATCH (a:IMGT_HLA)-[r:HAS_ALIGNMENT]-(n:GEN_ALIGN) " \
                + "WHERE a.locus = \"" + locus + "\" " \
                + "AND a.name = \"" + ref_allele + "\" " \
                + "AND r.imgt_release = \"" + imgtdb_version + "\" " \
                + "WITH [" + seq_l + "] AS RSEQ,n " \
                + "WITH [x in range(0,size(n.seq)-1) | x] AS ind,RSEQ,n " \
                + "UNWIND ind AS number " \
                + "WITH DISTINCT number,RSEQ,n " \
                + "WHERE NOT(n.seq[number] = RSEQ[number]) "\
                + "RETURN number, n.seq[number] AS REF, RSEQ[number] AS SEQ"
        diff_data = self.graph.run(query).to_data_frame()
        diffs = []
        for i in range(0, len(diff_data)):
            index = diff_data['number'][i]
            diff_feat = self.seqann[dbv].refdata.align_coordinates[locus.split("-")[1]][index]
            ref_seq = diff_data['REF'][i]
            in_seq = diff_data['SEQ'][i]
            relative_location = index + self.seqann[dbv].refdata.location[locus]
            # TODO: Change rank!
            diff = Seqdiff(term=diff_feat, location=str(relative_location), ref=ref_seq, inseq=in_seq)
            diffs.append(diff)
        return diffs

    def create_typing(self, similar_data, gfe, features):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """
        if not similar_data.empty:
            gfe_sim = {sim_gfe: self.calcSim(sim_gfe, gfe)
                       for sim_gfe in similar_data['GFE']}
            gfe_diff = {sim_gfe: self.calcDiff(sim_gfe, gfe)
                       for sim_gfe in similar_data['GFE']}
            hla_sim = {similar_data['GFE'][i]: similar_data['HLA'][i]
                       for i in range(0, len(similar_data['GFE']))}
            max_val = max(gfe_sim.values())
            min_val = min(gfe_diff.values())

            # TODO: Add more complex logic here about 
            max_gfes = [[g, hla_sim[g]] for g in gfe_sim.keys() if gfe_sim[g] == max_val]
            min_gfes = [[g, hla_sim[g]] for g in gfe_sim.keys() if gfe_diff[g] == min_val]
            diff_max = [g[0] for g in max_gfes if gfe_diff[g[0]] > 0]
            if len(diff_max) == len(max_gfes) and len(min_gfes) == 1 and min_val == 0:
                max_gfes = min_gfes
            hla_list = []
            gfe_list = []
            for gfes_hla in max_gfes:
                gfes, hla = gfes_hla[0], gfes_hla[1]
                hla_list.append(hla)
                gfe_list.append(gfes)
            return ["/".join(hla_list), "/".join(gfe_list)]
        else:
            return list()

    def find_gfe_kir(self, gfe, features):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """
        [locus, feature_accessions] = gfe.split("w")
        cypher = similar_kir(locus)
        similar_data = self.graph.run(cypher).to_data_frame()
        return self.create_typing(similar_data, gfe, features)

    def map_structures(self, gfe_structs):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """
        feat_map = {}
        for feat in gfe_structs:
            feat_name = "-".join([feat.term.upper(), str(feat.rank)])
            feat_map.update({feat_name: feat.sequence})
        return feat_map

    def matching_features(self, gfe1, gfe2, structures):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """

        # TODO: Update looping
        gfe_parts1 = self.breakup_gfe(gfe1)
        gfe_parts2 = self.breakup_gfe(gfe2)
        feat_list = list()
        for feat in gfe_parts1:
            if feat in gfe_parts2:
                if gfe_parts1[feat] == gfe_parts2[feat]:
                    if not feat == "FIVE_PRIME_UTR" and not feat == "THREE_PRIME_UTR":
                        feat_term, feat_rank = feat.split('-')
                        shared_feat = Feature(term=feat_term.upper(), rank=feat_rank, sequence=structures[feat], accession=gfe_parts1[feat])
                        feat_list.append(shared_feat)
                    else:
                        shared_feat = Feature(term=feat, rank=1, sequence=structures["-".join([feat, str(1)])], accession=gfe_parts1["-".join([feat, str(1)])])
                        feat_list.append(shared_feat)
        return(feat_list)

    def breakup_gfe(self, gfe):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """
        [locus, feature_accessions] = gfe.split("w")
        accessions = feature_accessions.split("-")
        i = 0
        features = {}
        for feature_rank in self.structures[locus]:
            accession = accessions[i]
            if feature_rank.upper() == "FIVE_PRIME_UTR" or feature_rank.upper() == "THREE_PRIME_UTR":
                features.update({"_".join([feature_rank, str(1)]).upper(): accession})
            else:
                features.update({feature_rank.upper(): accession})
            i += 1
        return(features)

    def calcSim(self, gfe1, gfe2):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """
        count = 0
        gfe1_parts = gfe1.split("w")[1].split("-")
        gfe2_parts = gfe2.split("w")[1].split("-")
        for i in range(0, len(gfe1_parts)):
            if gfe1_parts[i] == gfe2_parts[i] and \
                    not str(gfe1_parts[i]) == str(0):
                count += 1

        return count

    def calcDiff(self, gfe1, gfe2):
        """
        creates GFE from HLA sequence and locus

        :param locus: string containing HLA locus.
        :param sequence: string containing sequence data.

        :return: GFEobject.
        """
        count = 0
        gfe1_parts = gfe1.split("w")[1].split("-")
        gfe2_parts = gfe2.split("w")[1].split("-")
        for i in range(0, len(gfe1_parts)):
            if gfe1_parts[i] != gfe2_parts[i] and \
                    not str(gfe1_parts[i]) == str(0):
                count += 1

        return count

    def list_all_db_releases(self):
        cypher = all_db_imgt()
        response_data = self.graph.run(cypher).to_data_frame()
        return list(response_data.HLA_DB)


class SeqAnnException(Exception):

    def __init__(self, inputtype=None, reason=None):
        self.inputtype = inputtype
        self.reason = reason

    def __str__(self):
        """
        Custom error messages for exception
        """
        error_message = "Reason: ({0})\n"\
                        "Type: {1}\n".format(self.reason, self.inputtype)

        return error_message



#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
#    pygfe pyGFE.
#    Copyright (c) 2017 Be The Match operated by National Marrow Donor Program. All Rights Reserved.
#
#    This library is free software; you can redistribute it and/or modify it
#    under the terms of the GNU Lesser General Public License as published
#    by the Free Software Foundation; either version 3 of the License, or (at
#    your option) any later version.
#
#    This library is distributed in the hope that it will be useful, but WITHOUT
#    ANY WARRANTY; with out even the implied warranty of MERCHANTABILITY or
#    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
#    License for more details.
#
#    You should have received a copy of the GNU Lesser General Public License
#    along with this library;  if not, write to the Free Software Foundation,
#    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.
#
#    > http://www.fsf.org/licensing/licenses/lgpl.html
#    > http://www.opensource.org/licenses/lgpl-license.php
#

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

with open('requirements.txt') as reqs:
    requirements = reqs.read().split()

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='py-gfe',
    version='1.1.3',
    description="Python package for converting sequence annotations to gene feature enumerations (GFE).",
    long_description=readme + '\n\n' + history,
    author="CIBMTR",
    author_email='cibmtr-pypi@nmdp.org',
    url='https://github.com/nmdp-bioinformatics/py-gfe',
    packages=[
        'pygfe',
        'pygfe.models',
        'pygfe.feature_client',
        'pygfe.feature_client.apis',
        'pygfe.feature_client.models'
    ],
    package_dir={'pygfe':
                 'pygfe'},
    install_requires=requirements,
    license="LGPL 3.0",
    zip_safe=False,
    keywords='pygfe',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    scripts=['scripts/seq2gfe'],
    test_suite='tests',
    tests_require=test_requirements
)
